//---------------------------------------------------------------------------
// ExecFiles.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "Lib/Moochos/Moa/Shell.h"
#include "ExecFiles.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Execian { namespace ExecFiles {
//---------------------------------------------------------------------------
void __fastcall TExecApp::Initialize()
{
	try {
		FIcon = new TIcon();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecApp::Uninitialize()
{
	delete FIcon;
	FIcon = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TExecApp::Derefer(const AnsiString& AFileName)
{
	try {
		TPtr<TExecApp> instance = new TExecApp();
		{
			TPtr<TShellLink> shellLink = new TShellLink();
			shellLink->LoadFromFile(AFileName);
			instance->DereferIcon(AFileName, instance->Icon);
			instance->Title = ChangeFileExt(ExtractFileName(AFileName), AnsiString());
			instance->FileName = shellLink->TargetFile;
			instance->WorkingDir = shellLink->WorkingDir;
			instance->WindowState = shellLink->ShowCommand;
		}
		Assign(instance);
	}
	catch(EShellLinkError& e) {
		throw EExecFileError("The program cannot load an application from the shortcut file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TExecApp::DereferIcon(const AnsiString& AFileName, TIcon* AIcon) const
{
	AIcon->Handle = NULL;
	SHFILEINFO fileInfo = {0};
	HIMAGELIST imageList = reinterpret_cast<HIMAGELIST>(SHGetFileInfo(
		AFileName.c_str(),
		0,
		&fileInfo,
		sizeof(SHFILEINFO),
		SHGFI_SMALLICON | SHGFI_SYSICONINDEX));
	if(imageList == NULL) {
		return;
	}
	HICON iconHandle = ImageList_GetIcon(
		imageList,
		fileInfo.iIcon,
		ILD_TRANSPARENT);
	if(iconHandle == NULL) {
		return;
	}
	AIcon->Handle = iconHandle;
}
//---------------------------------------------------------------------------
__fastcall TExecApp::TExecApp() :
	TObject(),
	FIcon(NULL),
	FTitle(),
	FFileName(),
	FWindowState(SW_SHOWNORMAL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TExecApp::TExecApp(const AnsiString& AShortcutFile) :
	TObject(),
	FIcon(NULL),
	FTitle(),
	FFileName(),
	FWindowState(SW_SHOWNORMAL)
{
	try {
		Initialize();
		Derefer(AShortcutFile);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
__fastcall TExecApp::~TExecApp()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TExecApp::Assign(TExecApp* AOther)
{
	if(AOther == this) {
		return;
	}
	FIcon->Assign(AOther->FIcon);
	FTitle = AOther->FTitle;
	FFileName = AOther->FFileName;
	FWorkingDir = AOther->FWorkingDir;
	FWindowState = AOther->FWindowState;
}
//---------------------------------------------------------------------------
void __fastcall TExecDoc::SetFileName(const AnsiString& AFileName)
{
	FFileName = AFileName;
	FName = ExtractFileName(FileName);
	FDirectory = ExtractFileDir(FileName);
}
//---------------------------------------------------------------------------
__fastcall TExecDoc::TExecDoc() :
	TObject(),
	FFileName(),
	FName(),
	FDirectory(),
	FResult(drNoResult),
	FError(deNoError)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TExecDoc::TExecDoc(const AnsiString& AFileName) :
	TObject(),
	FFileName(),
	FName(),
	FDirectory(),
	FResult(drNoResult),
	FError(deNoError)
{
	FileName = AFileName;
}
//---------------------------------------------------------------------------
__fastcall TExecDoc::~TExecDoc()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TExecDoc::Assign(TExecDoc* AOther)
{
	if(AOther == this) {
		return;
	}
	FFileName = AOther->FFileName;
	FName = AOther->FName;
	FDirectory = AOther->FDirectory;
	FResult = AOther->FResult;
	FError = AOther->FError;
}
//---------------------------------------------------------------------------
AnsiString __fastcall ExecDocResultToStr(TExecDocResult AResult)
{
	static const AnsiString ExecDocResultStrings[] =
	{
		"",
		"Success",
		"Failure",
	};

	return ExecDocResultStrings[static_cast<int>(AResult)];
}
//---------------------------------------------------------------------------
AnsiString __fastcall ExecDocErrorToStr(TExecDocError AError)
{
	static const AnsiString ExecDocErrorStrings[] =
	{
		"",
		"File not found",
		"File name not found",
		"DDE failed",
		"No accociation",
		"Access denied",
		"DLL not found",
		"Canceled",
		"Not enough memory",
		"Sharing violation",
		"Unknown",
	};

	return ExecDocErrorStrings[static_cast<int>(AError)];
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Execian::ExecFiles
//---------------------------------------------------------------------------

