//---------------------------------------------------------------------------
#ifndef Moochos_Execian_ExecTask_H
#define Moochos_Execian_ExecTask_H
//---------------------------------------------------------------------------
// ExecTask.h
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include "Lib/Moochos/Moa/Tasks.h"
#include "ExecFiles.h"
//---------------------------------------------------------------------------
namespace Moochos { namespace Execian { namespace ExecTask {
//---------------------------------------------------------------------------
class EExecExecuteError : public Exception
{
public:
	__fastcall EExecExecuteError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class EExecWaitError : public Exception
{
public:
	__fastcall EExecWaitError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TExecTask : public TTask
{
private:
	class EExecTaskCancel : public Exception
	{
	public:
		__fastcall EExecTaskCancel(const AnsiString& AMessage) : Exception(AMessage) {}
	};
	TExecApp* FApp;
	DynamicArray<TExecDoc*> FDocs;
	TExecApp* FCurApp;
	TExecDoc* FCurDoc;
	int FCurrent;
	bool FSkippedWait;
	bool FPaused;
	TNotifyEvent FOnSkipWait;
	TNotifyEvent FOnPause;
	TNotifyEvent FOnResume;
	TNotifyEvent FOnAllReset;
	TNotifyEvent FOnOneExecute;
	TNotifyEvent FOnOneResult;
	TNotifyEvent FOnOneWait;
	TNotifyEvent FOnOneWaitFail;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall ResetDocs();
	void __fastcall UpdateCurFiles();
	void __fastcall UpdateCurFilesInMainThread(void* AData);
	void __fastcall UpdateOriFiles();
	void __fastcall UpdateOriFilesInMainThread(void* AData);
	void __fastcall EnsureNotPaused();
	void __fastcall EnsureNotCanceled();
	void __fastcall CommandExecute(TObject* ASender);
	void __fastcall CommandResult(TObject* ASender);
	void __fastcall CommandWait(TObject* ASender);
	void __fastcall CommandCanWaitMore(TObject* ASender, bool& ACanWaitMore);
	void __fastcall DoSkipWait();
	void __fastcall DoSkipWaitInMainThread(void* AData);
	void __fastcall DoPause();
	void __fastcall DoPauseInMainThread(void* AData);
	void __fastcall DoResume();
	void __fastcall DoResumeInMainThread(void* AData);
	void __fastcall DoAllReset();
	void __fastcall DoAllResetInMainThread(void* AData);
	void __fastcall DoOneExecute();
	void __fastcall DoOneExecuteInMainThread(void* AData);
	void __fastcall DoOneResult();
	void __fastcall DoOneResultInMainThread(void* AData);
	void __fastcall DoOneWait();
	void __fastcall DoOneWaitInMainThread(void* AData);
	void __fastcall DoOneWaitFail();
	void __fastcall DoOneWaitFailInMainThread(void* AData);
	TExecDoc* __fastcall GetDoc(int AIndex);
	int __fastcall GetAll();
protected:
	virtual void __fastcall Main();
public:
	__fastcall TExecTask(TExecApp* AApp, DynamicArray<TExecDoc*> ADocs);
	virtual __fastcall ~TExecTask();
	void __fastcall SkipWait();
	void __fastcall Pause();
	void __fastcall Resume();
	__property TExecApp* App = {read=FApp};
	__property TExecDoc* Docs[int AIndex] = {read=GetDoc};
	__property int Current = {read=FCurrent};
	__property int All = {read=GetAll};
	__property bool SkippedWait = {read=FSkippedWait};
	__property bool Paused = {read=FPaused};
	__property TNotifyEvent OnSkipWait = {read=FOnSkipWait, write=FOnSkipWait};
	__property TNotifyEvent OnPause = {read=FOnPause, write=FOnPause};
	__property TNotifyEvent OnResume = {read=FOnResume, write=FOnResume};
	__property TNotifyEvent OnAllReset = {read=FOnAllReset, write=FOnAllReset};
	__property TNotifyEvent OnOneExecute = {read=FOnOneExecute, write=FOnOneExecute};
	__property TNotifyEvent OnOneResult = {read=FOnOneResult, write=FOnOneResult};
	__property TNotifyEvent OnOneWait = {read=FOnOneWait, write=FOnOneWait};
	__property TNotifyEvent OnOneWaitFail = {read=FOnOneWaitFail, write=FOnOneWaitFail};
};
//---------------------------------------------------------------------------
class EExecCommandError : public Exception
{
public:
	__fastcall EExecCommandError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TExecCommand : public TObject
{
public:
	typedef void __fastcall (__closure * TCanWaitMoreEvent)(TObject* ASender, bool& ACanWaitMore);
private:
	TExecApp* FApp;
	TExecDoc* FDoc;
	int FWaitInterval;
	TNotifyEvent FOnExecute;
	TNotifyEvent FOnResult;
	TNotifyEvent FOnWait;
	TCanWaitMoreEvent FOnCanWaitMore;
	void __fastcall Main();
	HANDLE __fastcall MainExecute();
	void __fastcall MainWait(HANDLE AProcess);
	void __fastcall DoExecute();
	void __fastcall DoResult();
	void __fastcall DoWait();
	bool __fastcall DoCanWaitMore();
	TExecDocError __fastcall OsErrorToExecDocError(DWORD AOsError);
protected:
public:
	__fastcall TExecCommand(TExecApp* AApp, TExecDoc* ADoc);
	virtual __fastcall ~TExecCommand();
	void __fastcall Execute();
	__property TExecApp* App = {read=FApp};
	__property TExecDoc* Doc = {read=FDoc};
	__property int WaitInterval = {read=FWaitInterval, write=FWaitInterval};
	__property TNotifyEvent OnExecute = {read=FOnExecute, write=FOnExecute};
	__property TNotifyEvent OnResult = {read=FOnResult, write=FOnResult};
	__property TNotifyEvent OnWait = {read=FOnWait, write=FOnWait};
	__property TCanWaitMoreEvent OnCanWaitMore = {read=FOnCanWaitMore, write=FOnCanWaitMore};
};
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Execian::ExecTask
using namespace Moochos::Execian::ExecTask;
//---------------------------------------------------------------------------
#endif

