//---------------------------------------------------------------------------
// Clipboard.cpp
//
// Version: 1.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Clipboard.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Clipboard {
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Initialize()
{
	try {
		FHandle = AllocateHWnd(HandleMessage);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Uninitialize()
{
	if(FHandle != NULL) {
		DeallocateHWnd(FHandle);
		FHandle = NULL;
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::HandleMessage(TMessage& AMessage)
{
	switch(AMessage.Msg) {
	case WM_CHANGECBCHAIN:
		WMChangeCbChain(AMessage);
		break;
	case WM_DRAWCLIPBOARD:
		WMDrawClipboard(AMessage);
		break;
	default:
		AMessage.Result = DefWindowProc(FHandle, AMessage.Msg, AMessage.WParam, AMessage.LParam);
		break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::WMChangeCbChain(TMessage& AMessage)
{
	if(reinterpret_cast<HWND>(AMessage.WParam) == FNextHandle) {
		FNextHandle = reinterpret_cast<HWND>(AMessage.LParam);
		return;
	}
	if(FNextHandle == NULL) {
		return;
	}
	SendMessage(FNextHandle, AMessage.Msg, AMessage.WParam, AMessage.LParam);
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::WMDrawClipboard(TMessage& AMessage)
{
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::SetEnabled(bool AEnabled)
{
	if((AEnabled ? 1 : 0) == (FEnabled ? 1 : 0)) {
		return;
	}
	if(AEnabled) {
		::SetLastError(NO_ERROR);
		FNextHandle = SetClipboardViewer(FHandle);
		if(::GetLastError() != NO_ERROR) {
			throw EClipboardError("The program cannot add the clipboard viewer to the chain.");
		}
	}
	else {
		ChangeClipboardChain(FHandle, FNextHandle); // No way to determine success
		FNextHandle = NULL;
	}
	FEnabled = AEnabled;
}
//---------------------------------------------------------------------------
__fastcall TClipboardWatcher::TClipboardWatcher() :
	TObject(),
	FHandle(NULL),
	FNextHandle(NULL),
	FEnabled(false),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TClipboardWatcher::~TClipboardWatcher()
{
	Enabled = false;
	Uninitialize();
}
//---------------------------------------------------------------------------
}}} // End of Moochos::Moa::Clipboard
//---------------------------------------------------------------------------

