//---------------------------------------------------------------------------
// ListControls.cpp
//
// Version: 1.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <mbstring.h>
#include <Math.hpp>
#include <UtilCls.h>
#include "ListControls.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace ListControls {
//---------------------------------------------------------------------------
const int lsCut = 1 << 0;
const int lsDropTarget = 1 << 1;
const int lsFocused = 1 << 2;
const int lsSelected = 1 << 3;
//---------------------------------------------------------------------------
int __fastcall TListControlShell::SortListCompare(void* AItem1, void* AItem2)
{
	TSortItem* sortItem1 = static_cast<TSortItem*>(AItem1);
	TSortItem* sortItem2 = static_cast<TSortItem*>(AItem2);
	return sortItem1->CompareProc(sortItem1->Index, sortItem2->Index, sortItem1->CompareData);
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::Expand()
{
	Capacity = Capacity == 0 ? 1 : (Capacity * 2);
}
//---------------------------------------------------------------------------
int __fastcall TListControlShell::GetState(int AIndex)
{
	return FStates[AIndex];
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::SetState(int AIndex, int AState)
{
	int oldState = States[AIndex];
	if(AState == oldState) {
		return;
	}
	if((oldState & lsFocused) != 0) {
		FFocusIndex = -1;
	}
	if((oldState & lsSelected) != 0) {
		FSelCount--;
	}
	int newState = FStates[AIndex] = AState;
	if((newState & lsFocused) != 0) {
		if(FocusIndex >= 0) {
			FStates[FocusIndex] &= ~lsFocused;
		}
		FFocusIndex = AIndex;
	}
	if((newState & lsSelected) != 0) {
		FSelCount++;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::SetCount(int ACount)
{
	if(ACount == Count) {
		return;
	}
	if(ACount == 0) {
		FStates.Length = 0;
		FCount = 0;
		FFocusIndex = -1;
		FSelCount = 0;
	}
	else if(ACount < Count) {
		FocusIndex = EnsureRange(FocusIndex, -1, ACount - 1);
		for(int i = 0; i < Count - ACount; i++) {
			States[ACount + i] = 0;
		}
		FCount = ACount;
	}
	else {
		while(Capacity < ACount) {
			Expand();
		}
		for(int i = 0; i < ACount - Count; i++) {
			FStates[Count + i] = 0;
		}
		FCount = ACount;
	}
}
//---------------------------------------------------------------------------
int __fastcall TListControlShell::GetCapacity()
{
	return FStates.Length;
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::SetCapacity(int ACapacity)
{
	FStates.Length = ACapacity;
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::SetFocusIndex(int AFocusIndex)
{
	if(AFocusIndex == FocusIndex || (AFocusIndex < 0 && FocusIndex < 0)) {
		return;
	}
	if(FocusIndex >= 0) {
		States[FocusIndex] = States[FocusIndex] & ~lsFocused;
	}
	if(AFocusIndex >= 0) {
		States[AFocusIndex] = States[AFocusIndex] | lsFocused;
	}
}
//---------------------------------------------------------------------------
TBooleanDynArray __fastcall TListControlShell::GetSelTable()
{
	TBooleanDynArray selTable;
	selTable.Length = Count;
	for(int i = 0; i < Count; i++) {
		selTable[i] = (States[i] & lsSelected) != 0;
	}
	return selTable;
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::SetSelTable(const TBooleanDynArray& ASelTable)
{
	SelectItems(ASelTable);
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TListControlShell::MakeOrder(bool AAscending)
{
	TIntegerDynArray order;
	order.Length = Count;
	for(int i = 0; i < Count; i++) {
		order[i] = AAscending ? i : (Count - 1 - i);
	}
	return order;
}
//---------------------------------------------------------------------------
__fastcall TListControlShell::TListControlShell(TListControlShellView* AView) :
	TObject(),
	FView(AView),
	FStates(),
	FCount(0),
	FFocusIndex(-1),
	FSelCount(0)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TListControlShell::~TListControlShell()
{
	delete FView;
}
//---------------------------------------------------------------------------
int __fastcall TListControlShell::Add()
{
	int addIndex = Count;
	Insert(addIndex);
	return addIndex;
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::Insert(int AIndex)
{
	Count = Count + 1;
	Move(Count - 1, AIndex);
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::Delete(int AIndex)
{
	if(FocusIndex == AIndex) {
		FocusIndex = EnsureRange(FocusIndex + 1, -1, Count - 1 - 1);
	}
	Move(AIndex, Count - 1);
	Count = Count - 1;
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::DeleteSelected()
{
	DeleteItems(SelTable);
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::DeleteItems(const TBooleanDynArray& ACanDelete)
{
	int oldFocusIndex = FocusIndex;
	int newFocusIndex = oldFocusIndex;
	int deleteCount = 0;
	for(int i = 0; i < Count; i++) {
		if(ACanDelete[i]) {
			deleteCount++;
			if(oldFocusIndex > i) {
				newFocusIndex--;
			}
		}
		else {
			States[i - deleteCount] = States[i];
		}
	}
	Count = Count - deleteCount;
	FocusIndex = EnsureRange(newFocusIndex, -1, Count - 1);
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::Clear()
{
	Count = 0;
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::SelectAll()
{
	for(int i = 0; i < Count; i++) {
		States[i] = States[i] | lsSelected;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::ClearSelection()
{
	for(int i = 0; i < Count; i++) {
		States[i] = States[i] & ~lsSelected;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::ReverseSelection()
{
	for(int i = 0; i < Count; i++) {
		States[i] = States[i] ^ lsSelected;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::SelectItems(const TBooleanDynArray& ACanSelect)
{
	for(int i = 0; i < Count; i++) {
		States[i] = ACanSelect[i] ? (States[i] | lsSelected) : (States[i] & ~lsSelected);
	}
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::Exchange(int AIndex1, int AIndex2)
{
	if(AIndex1 == AIndex2) {
		return;
	}
	int temp = States[AIndex1];
	States[AIndex1] = States[AIndex2];
	States[AIndex2] = temp;
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::Move(int AOldIndex, int ANewIndex)
{
	if(AOldIndex == ANewIndex) {
		return;
	}
	int temp = States[AOldIndex];
	int exchangeCount = AOldIndex < ANewIndex ? (ANewIndex - AOldIndex) : (AOldIndex - ANewIndex);
	for(int i = 0; i < exchangeCount; i++) {
		int oldIndex = AOldIndex < ANewIndex ? (AOldIndex + 1 + i) : (AOldIndex - 1 - i);
		int newIndex = AOldIndex < ANewIndex ? (oldIndex - 1) : (oldIndex + 1);
		States[newIndex] = States[oldIndex];
	}
	States[ANewIndex] = temp;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TListControlShell::MoveSelected(int ADistance)
{
	TIntegerDynArray order = MakeOrder(true);
	{
		int moveRest = (ADistance < 0 ? -ADistance : ADistance) + 1;
		while(--moveRest > 0) {
			for(int i = 0; i < Count; i++) {
				if(i == 0) {
					continue;
				}
				int oldIndex = ADistance < 0 ? i : (Count - 1 - i);
				int newIndex = ADistance < 0 ? (oldIndex - 1) : (oldIndex + 1);
				if((States[order[oldIndex]] & lsSelected) == 0 || (States[order[newIndex]] & lsSelected) != 0) {
					continue;
				}
				int temp = order[oldIndex];
				order[oldIndex] = order[newIndex];
				order[newIndex] = temp;
			}
		}
	}
	Reorder(order);
	return order;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TListControlShell::Sort(TSortCompareProc ACompareProc, void* ACompareData)
{
	TIntegerDynArray order = MakeOrder(true);
	{
		DynamicArray<TSortItem> sortItems;
		sortItems.Length = Count;
		for(int i = 0; i < Count; i++) {
			sortItems[i].Index = i;
			sortItems[i].CompareProc = ACompareProc;
			sortItems[i].CompareData = ACompareData;
		}

		TPtr<TList> sortList = new TList();
		sortList->Count = Count;
		for(int i = 0; i < Count; i++) {
			sortList->Items[i] = &sortItems[i];
		}
		sortList->Sort(SortListCompare);

		order.Length = Count;
		for(int i = 0; i < Count; i++) {
			order[i] = static_cast<TSortItem*>(sortList->Items[i])->Index;
		}
	}
	Reorder(order);
	return order;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TListControlShell::Invert()
{
	TIntegerDynArray order = MakeOrder(false);
	Reorder(order);
	return order;
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::Reorder(const TIntegerDynArray& AOrder)
{
	TIntegerDynArray oldStates = FStates.Copy();
	for(int i = 0; i < Count; i++) {
		States[i] = oldStates[AOrder[i]];
	}
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::Update()
{
	Clear();
	Count = View->Count;
	for(int i = 0; i < Count; i++) {
		States[i] = View->States[i];
	}
}
//---------------------------------------------------------------------------
void __fastcall TListControlShell::UpdateControl()
{
	View->BeginUpdate();
	try {
		View->Count = Count;
		for(int i = 0; i < Count; i++) {
			View->States[i] = States[i];
		}
	}
	__finally {
		View->EndUpdate();
	}
}
//---------------------------------------------------------------------------
__fastcall TListControlShellView::TListControlShellView(TWinControl* AControl) :
	TObject(),
	FControl(AControl)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TListControlShellView::~TListControlShellView()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TListView* __fastcall TListViewShellView::GetControl()
{
	return static_cast<TListView*>(TListControlShellView::Control);
}
//---------------------------------------------------------------------------
int __fastcall TListViewShellView::GetState(int AIndex)
{
	int state = 0;
	UINT controlState = ListView_GetItemState(
		Control->Handle,
		AIndex,
		LVIS_CUT | LVIS_DROPHILITED | LVIS_FOCUSED | LVIS_SELECTED);
	if((controlState & LVIS_CUT) != 0) {
		state |= lsCut;
	}
	if((controlState & LVIS_DROPHILITED) != 0) {
		state |= lsDropTarget;
	}
	if((controlState & LVIS_FOCUSED) != 0) {
		state |= lsFocused;
	}
	if((controlState & LVIS_SELECTED) != 0) {
		state |= lsSelected;
	}
	return state;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShellView::SetState(int AIndex, int AState)
{
	UINT controlState = 0;
	if((AState & lsCut) != 0) {
		controlState |= LVIS_CUT;
	}
	if((AState & lsDropTarget) != 0) {
		controlState |= LVIS_DROPHILITED;
	}
	if((AState & lsFocused) != 0) {
		controlState |= LVIS_FOCUSED;
	}
	if((AState & lsSelected) != 0) {
		controlState |= LVIS_SELECTED;
	}
	ListView_SetItemState(
		Control->Handle,
		AIndex,
		controlState,
		LVIS_CUT | LVIS_DROPHILITED | LVIS_FOCUSED | LVIS_SELECTED);
}
//---------------------------------------------------------------------------
int __fastcall TListViewShellView::GetCount()
{
	return Control->Items->Count;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShellView::SetCount(int ACount)
{
	Control->Items->Count = ACount;
}
//---------------------------------------------------------------------------
int __fastcall TListViewShellView::GetFocusIndex()
{
	if(Control->ItemFocused == NULL) {
		return -1;
	}
	return Control->ItemFocused->Index;
}
//---------------------------------------------------------------------------
void __fastcall TListViewShellView::SetFocusIndex(int AFocusIndex)
{
	if(AFocusIndex < 0) {
		if(Control->ItemFocused == NULL) {
			return;
		}
		Control->ItemFocused->Focused = false;
	}
	else {
		Control->Items->Item[AFocusIndex]->Focused = true;
	}
}
//---------------------------------------------------------------------------
int __fastcall TListViewShellView::GetSelCount()
{
	return Control->SelCount;
}
//---------------------------------------------------------------------------
__fastcall TListViewShellView::TListViewShellView(TListView* AControl) :
	TListControlShellView(AControl)
{
	// Do nothin
}
//---------------------------------------------------------------------------
__fastcall TListViewShellView::~TListViewShellView()
{
	// Do nothin
}
//---------------------------------------------------------------------------
void __fastcall TListViewShellView::BeginUpdate()
{
	Control->Items->BeginUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TListViewShellView::EndUpdate()
{
	Control->Items->EndUpdate();
}
//---------------------------------------------------------------------------
TListBox* __fastcall TListBoxShellView::GetControl()
{
	return static_cast<TListBox*>(TListControlShellView::Control);
}
//---------------------------------------------------------------------------
int __fastcall TListBoxShellView::GetState(int AIndex)
{
	int state = 0;
	if(AIndex == ::SendMessage(Control->Handle, LB_GETCARETINDEX, 0, 0)) {
		state |= lsFocused;
	}
	if(::SendMessage(Control->Handle, LB_GETSEL, AIndex, 0)) {
		state |= lsSelected;
	}
	return state;
}
//---------------------------------------------------------------------------
void __fastcall TListBoxShellView::SetState(int AIndex, int AState)
{
	if((AState & lsFocused) != 0) {
		::SendMessage(Control->Handle, LB_SETCARETINDEX, AIndex, MAKELPARAM(true, 0));
	}
	::SendMessage(Control->Handle, LB_SETSEL, (AState & lsSelected) != 0, AIndex);
}
//---------------------------------------------------------------------------
int __fastcall TListBoxShellView::GetCount()
{
	return Control->Count;
}
//---------------------------------------------------------------------------
void __fastcall TListBoxShellView::SetCount(int ACount)
{
	Control->Count = ACount;
}
//---------------------------------------------------------------------------
int __fastcall TListBoxShellView::GetFocusIndex()
{
	return ::SendMessage(Control->Handle, LB_GETCARETINDEX, 0, 0);
}
//---------------------------------------------------------------------------
void __fastcall TListBoxShellView::SetFocusIndex(int AFocusIndex)
{
	::SendMessage(Control->Handle, LB_SETCARETINDEX, AFocusIndex, MAKELPARAM(true, 0));
}
//---------------------------------------------------------------------------
int __fastcall TListBoxShellView::GetSelCount()
{
	return ::SendMessage(Control->Handle, LB_GETSELCOUNT, 0, 0);
}
//---------------------------------------------------------------------------
__fastcall TListBoxShellView::TListBoxShellView(TListBox* AControl) :
	TListControlShellView(AControl)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TListBoxShellView::~TListBoxShellView()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TListBoxShellView::BeginUpdate()
{
	Control->Items->BeginUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TListBoxShellView::EndUpdate()
{
	Control->Items->EndUpdate();
}
//---------------------------------------------------------------------------
int __fastcall TListItemLayout::GetItemHeight()
{
	return FVertPadding * 2 + (FIconHeight > FTextHeight ? FIconHeight : FTextHeight);
}
//---------------------------------------------------------------------------
__fastcall TListItemLayout::TListItemLayout() :
	TObject(),
	FHorzPadding(0),
	FVertPadding(0),
	FIconWidth(0),
	FIconHeight(0),
	FTextHeight(0)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TListItemLayout::~TListItemLayout()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TRect __fastcall TListItemLayout::CalculateIconBounds(const TRect& ABounds)
{
	return Bounds(
		ABounds.left + HorzPadding,
		ABounds.top + (ItemHeight - IconHeight) / 2,
		IconWidth,
		IconHeight);
}
//---------------------------------------------------------------------------
TRect __fastcall TListItemLayout::CalculateTextBounds(const TRect& ABounds)
{
	return Bounds(
		ABounds.left + HorzPadding * 2 + IconWidth,
		ABounds.top + (ItemHeight - TextHeight) / 2,
		ABounds.Width() - HorzPadding * 3 - IconWidth,
		TextHeight);
}
//---------------------------------------------------------------------------
void __fastcall SelectItems(TListBox* AControl, int AFirstIndex, int ALastIndex, bool ASelect)
{
	if(AFirstIndex == ALastIndex) {
		SendMessage(AControl->Handle, LB_SETSEL, ASelect, AFirstIndex);
	}
	else if(AFirstIndex < ALastIndex) {
		if(ASelect) {
			SendMessage(AControl->Handle, LB_SELITEMRANGEEX, AFirstIndex, ALastIndex);
		}
		else {
			SendMessage(AControl->Handle, LB_SELITEMRANGEEX, ALastIndex, AFirstIndex);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall UpdateItems(TListBox* AControl, int AFirstIndex, int ALastIndex)
{
	TRect rect;
	if(!UnionRect(rect, AControl->ItemRect(AFirstIndex), AControl->ItemRect(ALastIndex))) {
		return;
	}
	InvalidateRect(AControl->Handle, &rect, false);
}
//---------------------------------------------------------------------------
void __fastcall MakeItemVisible(TListBox* AControl, int AIndex)
{
	TRect rect;
	if(IntersectRect(rect, AControl->ItemRect(AIndex), AControl->ClientRect)) {
		return;
	}
	AControl->TopIndex = AIndex;
}
//---------------------------------------------------------------------------
int __fastcall IncrementalFindCaption(TListBox* AControl, const AnsiString& AFindText)
{
	int itemCount = AControl->Items->Count;
	int itemIndex = (AControl->ItemIndex >= 0 ? AControl->ItemIndex : 0) - 1;
	for(int i = 0; i < itemCount; i++) {
		itemIndex = (itemIndex + 1) % itemCount;
		if(_mbsnicmp(
			AControl->Items->Strings[itemIndex].c_str(),
			AFindText.c_str(),
			AFindText.Length()) == 0) {
			return itemIndex;
		}
	}
	return -1;
}
//---------------------------------------------------------------------------
__fastcall TAutoListViewOnData::TAutoListViewOnData(TListView* AListView, TLVOwnerDataEvent AOnData) :
	FListView(AListView),
	FOnData(AListView->OnData)
{
	FListView->OnData = AOnData;
}
//---------------------------------------------------------------------------
__fastcall TAutoListViewOnData::~TAutoListViewOnData()
{
	FListView->OnData = FOnData;
}
//---------------------------------------------------------------------------
__fastcall TAutoListItemsUpdator::TAutoListItemsUpdator(TListItems* AListItems) :
	FListItems(AListItems)
{
	FListItems->BeginUpdate();
}
//---------------------------------------------------------------------------
__fastcall TAutoListItemsUpdator::~TAutoListItemsUpdator()
{
	FListItems->EndUpdate();
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::ListControls
//---------------------------------------------------------------------------

