//---------------------------------------------------------------------------
#ifndef Moochos_Moa_Shell_H
#define Moochos_Moa_Shell_H
//---------------------------------------------------------------------------
// Shell.h
//
// Version: 1.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Shell {
//---------------------------------------------------------------------------
class EShellError : public Exception
{
public:
	__fastcall EShellError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TFileDrop
	// Represents a drop operation of files.
{
private:
	TMessage FMessage;
	bool FAutoFinish;
	bool FFinished;
	__fastcall TFileDrop(const TFileDrop& AOther); // Forbidden
	TFileDrop& __fastcall operator =(const TFileDrop& AOther); // Forbidden
	void __fastcall EnsureNotFinished();
	HDROP __fastcall GetHandle();
	AnsiString __fastcall GetFile(int AIndex);
	int __fastcall GetCount();
	TPoint __fastcall GetPoint();
protected:
public:
	__fastcall TFileDrop(const TMessage& AMessage, bool AAutoFinish = true);
	__fastcall ~TFileDrop();
	void __fastcall GetFiles(TStrings* AFiles);
	void __fastcall Finish();
	__property HDROP Handle = {read=GetHandle};
	__property AnsiString Files[int AIndex] = {read=GetFile};
	__property int Count = {read=GetCount};
	__property TPoint Point = {read=GetPoint};
	__property bool AutoFinish = {read=FAutoFinish, write=FAutoFinish};
	__property bool Finished = {read=FFinished};
};
//---------------------------------------------------------------------------
class EShellLinkError : public Exception
{
public:
	__fastcall EShellLinkError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TShellLink : public TObject
	// Wraps the shell link.
{
private:
	IShellLink* FShellLink;
	IPersistFile* FPersistFile;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall InitializeInterfaces();
	void __fastcall UninitializeInterfaces();
	AnsiString __fastcall GetTargetFile();
	void __fastcall SetTargetFile(const AnsiString& ATargetFile);
	AnsiString __fastcall GetWorkingDir();
	void __fastcall SetWorkingDir(const AnsiString& AWorkingDir);
	AnsiString __fastcall GetArguments();
	void __fastcall SetArguments(const AnsiString& AArguments);
	AnsiString __fastcall GetDescription();
	void __fastcall SetDescription(const AnsiString& ADescription);
	AnsiString __fastcall GetIconFile();
	void __fastcall SetIconFile(const AnsiString& AIconFile);
	int __fastcall GetIconIndex();
	void __fastcall SetIconIndex(int AIconIndex);
	int __fastcall GetShowCommand();
	void __fastcall SetShowCommand(int AShowCommand);
	WORD __fastcall GetHotKey();
	void __fastcall SetHotKey(WORD AHotKey);
	LPITEMIDLIST __fastcall GetIdList();
	void __fastcall SetIdList(LPCITEMIDLIST AIdList);
	void __fastcall GetInError();
	void __fastcall SetInError();
protected:
public:
	__fastcall TShellLink();
	virtual __fastcall ~TShellLink();
	void __fastcall LoadFromFile(const AnsiString& AFileName);
	void __fastcall SaveToFile(const AnsiString& AFileName);
	void __fastcall Resolve(HWND AParent, DWORD AFlags = SLR_ANY_MATCH | SLR_UPDATE);
	void __fastcall GetIconLocation(AnsiString& AIconFile, int& AIconIndex);
	void __fastcall SetIconLocation(const AnsiString& AIconFile, int AIconIndex);
	__property AnsiString TargetFile = {read=GetTargetFile, write=SetTargetFile};
	__property AnsiString WorkingDir = {read=GetWorkingDir, write=SetWorkingDir};
	__property AnsiString Arguments = {read=GetArguments, write=SetArguments};
	__property AnsiString Description = {read=GetDescription, write=SetDescription};
	__property AnsiString IconFile = {read=GetIconFile, write=SetIconFile};
	__property int IconIndex = {read=GetIconIndex, write=SetIconIndex};
	__property int ShowCommand = {read=GetShowCommand, write=SetShowCommand};
	__property WORD HotKey = {read=GetHotKey, write=SetHotKey};
	__property LPITEMIDLIST IdList = {read=GetIdList, write=SetIdList};
	__property IShellLink* ShellLink = {read=FShellLink};
	__property IPersistFile* PersistFile = {read=FPersistFile};
};
//---------------------------------------------------------------------------
}}} // End of Moochos::Moa::Shell
using namespace Moochos::Moa::Shell;
//---------------------------------------------------------------------------
#endif

