//---------------------------------------------------------------------------
// ProgressDialogUnit.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "ProgressDialogUnit.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TProgressDialog* ProgressDialog = NULL;
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::Button91Click(TObject* ASender)
{
	Unmark();
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::Button92Click(TObject* ASender)
{
	SkipWait();
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::Button93Click(TObject* ASender)
{
	if(Paused) {
		Resume();
	}
	else {
		Pause();
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::Button94Click(TObject* ASender)
{
	Cancel();
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::WMClose(TMessage& AMessage)
{
	if(IsBusy()) {
		Cancel();
		return;
	}
	TForm::Dispatch(&AMessage);
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::WMExecute(TMessage& AMessage)
{
	if(!IsBusy()) {
		return;
	}
	DoExecute();
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::DoStart()
{
	if(OnStart != NULL) {
		OnStart(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::DoEnd()
{
	if(OnEnd != NULL) {
		OnEnd(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::DoExecute()
{
	if(OnExecute != NULL) {
		OnExecute(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::DoUnmark()
{
	if(OnUnmark != NULL) {
		OnUnmark(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::DoSkipWait()
{
	if(OnSkipWait != NULL) {
		OnSkipWait(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::DoPause()
{
	if(OnPause != NULL) {
		OnPause(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::DoResume()
{
	if(OnResume != NULL) {
		OnResume(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::DoCancel()
{
	if(OnCancel != NULL) {
		OnCancel(this);
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TProgressDialog::GetApplication()
{
	return Label112->Caption;
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::SetApplication(const AnsiString& AApplication)
{
	Label112->Caption = AApplication;
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::SetDocument(const AnsiString& ADocument)
{
	FDocument = ADocument;
	Label122->Caption = MinimizeName(Document, Label122->Canvas, Label122->Width);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TProgressDialog::GetStatus()
{
	return Label132->Caption;
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::SetStatus(const AnsiString& AStatus)
{
	Label132->Caption = AStatus;
}
//---------------------------------------------------------------------------
int __fastcall TProgressDialog::GetProgress()
{
	return ProgressBar11->Position;
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::SetProgress(int AProgress)
{
	ProgressBar11->Position = AProgress;
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::SetNoResult(int ANoResult)
{
	FNoResult = ANoResult;
	Label141->Caption = AnsiString().sprintf("No result: %d", NoResult);
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::SetSuccess(int ASuccess)
{
	FSuccess = ASuccess;
	Label142->Caption = AnsiString().sprintf("Success: %d", Success);
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::SetFailure(int AFailure)
{
	FFailure = AFailure;
	Label143->Caption = AnsiString().sprintf("Failure: %d", Failure);
}
//---------------------------------------------------------------------------
int __fastcall TProgressDialog::GetResultValue(TExecDocResult AResult)
{
	switch(AResult) {
	case drNoResult:
		return NoResult;
	case drSuccess:
		return Success;
	case drFailure:
		return Failure;
	default:
		throw EProgressDialogError("The result identifier is not supported.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::SetResultValue(TExecDocResult AResult, int AResultValue)
{
	switch(AResult) {
	case drNoResult:
		NoResult = AResultValue;
		break;
	case drSuccess:
		Success = AResultValue;
		break;
	case drFailure:
		Failure = AResultValue;
		break;
	default:
		throw EProgressDialogError("The result identifier is not supported.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TProgressDialog::GetStarted()
{
	return State.Contains(stStarted);
}
//---------------------------------------------------------------------------
bool __fastcall TProgressDialog::GetEnded()
{
	return State.Contains(stEnded);
}
//---------------------------------------------------------------------------
bool __fastcall TProgressDialog::GetPaused()
{
	return State.Contains(stPaused);
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::SetState(const TStates& AState)
{
	FState = AState;
	Button93->Caption = Paused ? "&Resume" : "&Pause";
	UpdateAccess();
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::UpdateAccess()
{
	Button91->Enabled = IsBusy();
	Button92->Enabled = IsBusy();
	Button93->Enabled = IsBusy();
	Button94->Enabled = IsBusy();
}
//---------------------------------------------------------------------------
__fastcall TProgressDialog::TProgressDialog(TComponent* AOwner) :
	TForm(AOwner),
	FDocument(),
	FNoResult(0),
	FSuccess(0),
	FFailure(0),
	FState(),
	FOnStart(NULL),
	FOnEnd(NULL),
	FOnExecute(NULL),
	FOnUnmark(NULL),
	FOnSkipWait(NULL),
	FOnPause(NULL),
	FOnResume(NULL),
	FOnCancel(NULL)
{
	NoResult = 0;
	Success = 0;
	Failure = 0;
	State = TStates();
}
//---------------------------------------------------------------------------
__fastcall TProgressDialog::~TProgressDialog()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::Execute()
{
	if(IsBusy()) {
		return;
	}
	PostMessage(Handle, WM_EXECUTE, 0, 0);
	State = TStates() << stStarted;
	SelectFirst();
	DoStart();
	ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::Finish()
{
	if(!IsBusy()) {
		return;
	}
	State = State >> stPaused << stEnded;
	DoEnd();
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::ForceClose()
{
	ModalResult = mrOk;
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::Unmark()
{
	if(!IsBusy()) {
		return;
	}
	DoUnmark();
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::SkipWait()
{
	if(!IsBusy()) {
		return;
	}
	DoSkipWait();
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::Pause()
{
	if(!IsBusy() || Paused) {
		return;
	}
	State = State << stPaused;
	DoPause();
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::Resume()
{
	if(!IsBusy() || !Paused) {
		return;
	}
	State = State >> stPaused;
	DoResume();
}
//---------------------------------------------------------------------------
void __fastcall TProgressDialog::Cancel()
{
	if(!IsBusy()) {
		return;
	}
	DoCancel();
}
//---------------------------------------------------------------------------
bool __fastcall TProgressDialog::IsBusy()
{
	return Started && !Ended;
}
//---------------------------------------------------------------------------
bool __fastcall TProgressDialog::IsCompleted()
{
	return Progress == 100;
}
//---------------------------------------------------------------------------

