//---------------------------------------------------------------------------
// SettingsMan.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "Lib/Moochos/Moa/Forms.h"
#include "Lib/Moochos/Moa/Hash.h"
#include "Lib/Moochos/Moa/IniFiles.h"
#include "Lib/Moochos/Moa/Sync.h"
#include "MainFormUnit.h"
#include "SettingsMan.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Execian { namespace SettingsMan {
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const AnsiString SettingsFileName = ExtractFilePath(Application->ExeName) + "Settings.ini";
const AnsiString SettingsLockName = AnsiString().sprintf(
	"Execian.Instance%d",
	TCrc32Checksum().Update(AnsiUpperCaseFileName(Application->ExeName)).Value);
const AnsiString idAddFilesDialog = "AddFilesDialog";
const AnsiString idAppsMan = "AppsMan";
const AnsiString idBoundsRect = "BoundsRect";
const AnsiString idColumns = "Columns";
const AnsiString idControl = "Control";
const AnsiString idCurrentDir = "CurrentDir";
const AnsiString idDocsMan = "DocsMan";
const AnsiString idExportDialog = "ExportDialog";
const AnsiString idFilterIndex = "FilterIndex";
const AnsiString idHideOnExecute = "HideOnExecute";
const AnsiString idImportDialog = "ImportDialog";
const AnsiString idItemIndex = "ItemIndex";
const AnsiString idItems = "Items";
const AnsiString idMainForm = "MainForm";
const AnsiString idSystem = "System";
const AnsiString idUseApp = "UseApp";
const AnsiString idWidth = "Width";
const AnsiString idWindowState = "WindowState";
const AnsiString icIndexBegin = "[";
const AnsiString icIndexEnd = "]";
const AnsiString icSeparator = ".";
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::LoadMainFormSection(TVarIniFile* AIniFile)
{
	TFormBounds(MainForm).NormalBounds = AIniFile->ReadRect(
		idMainForm,
		idBoundsRect,
		TFormBounds(MainForm).NormalBounds);

	MainForm->WindowState = static_cast<TWindowState>(AIniFile->ReadInteger(
		idMainForm,
		idWindowState,
		MainForm->WindowState));

	MainForm->UseApp = AIniFile->ReadBool(
		idMainForm,
		idUseApp,
		MainForm->UseApp);

	MainForm->HideOnExecute = AIniFile->ReadBool(
		idMainForm,
		idHideOnExecute,
		MainForm->HideOnExecute);

	MainForm->AppsMan->Control->ItemIndex = AIniFile->ReadInteger(
		idMainForm,
		idAppsMan + icSeparator + idControl + icSeparator + idItemIndex,
		MainForm->AppsMan->Control->ItemIndex);

	MainForm->DocsMan->AddFilesDialog->FilterIndex = AIniFile->ReadInteger(
		idMainForm,
		idDocsMan + icSeparator + idAddFilesDialog + icSeparator + idFilterIndex,
		MainForm->DocsMan->AddFilesDialog->FilterIndex);

	MainForm->DocsMan->ImportDialog->FilterIndex = AIniFile->ReadInteger(
		idMainForm,
		idDocsMan + icSeparator + idImportDialog + icSeparator + idFilterIndex,
		MainForm->DocsMan->ImportDialog->FilterIndex);

	MainForm->DocsMan->ExportDialog->FilterIndex = AIniFile->ReadInteger(
		idMainForm,
		idDocsMan + icSeparator + idExportDialog + icSeparator + idFilterIndex,
		MainForm->DocsMan->ExportDialog->FilterIndex);

	for(int i = 0; i < MainForm->DocsMan->Control->Columns->Count; i++) {
		MainForm->DocsMan->Control->Columns->Items[i]->Width = AIniFile->ReadInteger(
			idMainForm,
			idDocsMan + icSeparator + idControl + icSeparator + idColumns + icSeparator + idItems + icIndexBegin + i + icIndexEnd + icSeparator + idWidth,
			MainForm->DocsMan->Control->Columns->Items[i]->Width);
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::SaveMainFormSection(TVarIniFile* AIniFile)
{
	AIniFile->WriteRect(
		idMainForm,
		idBoundsRect,
		TFormBounds(MainForm).NormalBounds);

	AIniFile->WriteInteger(
		idMainForm,
		idWindowState,
		MainForm->WindowState);

	AIniFile->WriteBool(
		idMainForm,
		idUseApp,
		MainForm->UseApp);

	AIniFile->WriteBool(
		idMainForm,
		idHideOnExecute,
		MainForm->HideOnExecute);

	AIniFile->WriteInteger(
		idMainForm,
		idAppsMan + icSeparator + idControl + icSeparator + idItemIndex,
		MainForm->AppsMan->Control->ItemIndex);

	AIniFile->WriteInteger(
		idMainForm,
		idDocsMan + icSeparator + idAddFilesDialog + icSeparator + idFilterIndex,
		MainForm->DocsMan->AddFilesDialog->FilterIndex);

	AIniFile->WriteInteger(
		idMainForm,
		idDocsMan + icSeparator + idImportDialog + icSeparator + idFilterIndex,
		MainForm->DocsMan->ImportDialog->FilterIndex);

	AIniFile->WriteInteger(
		idMainForm,
		idDocsMan + icSeparator + idExportDialog + icSeparator + idFilterIndex,
		MainForm->DocsMan->ExportDialog->FilterIndex);

	for(int i = 0; i < MainForm->DocsMan->Control->Columns->Count; i++) {
		AIniFile->WriteInteger(
			idMainForm,
			idDocsMan + icSeparator + idControl + icSeparator + idColumns + icSeparator + idItems + icIndexBegin + i + icIndexEnd + icSeparator + idWidth,
			MainForm->DocsMan->Control->Columns->Items[i]->Width);
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::LoadSystemSection(TVarIniFile* AIniFile)
{
	SetCurrentDir(AIniFile->ReadQuotedString(
		idSystem,
		idCurrentDir,
		GetCurrentDir()));
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::SaveSystemSection(TVarIniFile* AIniFile)
{
	AIniFile->WriteQuotedString(
		idSystem,
		idCurrentDir,
		GetCurrentDir());
}
//---------------------------------------------------------------------------
__fastcall TSettingsMan::TSettingsMan() :
	TObject()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSettingsMan::~TSettingsMan()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::Load()
{
	if(!FileExists(SettingsFileName)) {
		return;
	}
	try {
		TPtr<TVarIniFile> iniFile = new TVarIniFile(SettingsFileName);
		LoadMainFormSection(iniFile);
		LoadSystemSection(iniFile);
	}
	catch(EIniFileException& e) {
		throw ESettingsManError("The program cannot load the settings.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::Save()
{
	TMutex lock(SettingsLockName, false);
	lock.Lock();

	try {
		TPtr<TVarIniFile> iniFile = new TVarIniFile(SettingsFileName);
		SaveMainFormSection(iniFile);
		SaveSystemSection(iniFile);
		iniFile->UpdateFile();
	}
	catch(EIniFileException& e) {
		throw ESettingsManError("The program cannot save the settings.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::DeleteFile(bool AConfirm)
{
	TMutex lock(SettingsLockName, false);
	lock.Lock();

	if(AConfirm) {
		if(Application->MessageBox(
			"Clear the settings?",
			Application->Title.c_str(),
			MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) != ID_YES) {
			Abort();
		}
	}
	if(!::DeleteFile(SettingsFileName.c_str())) {
		throw ESettingsManError("The program cannot delete the settings file.");
	}
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Execian:SettingsMan:
//---------------------------------------------------------------------------

